-- FuBar_AlarmClockFu 0.1.1
-- Author: Anthony White
-- Purpose: A simple alarm clock for FuBar
-- See Changelog.txt for revision information

AlarmClockFu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0")

local FMT_MIL = 0 -- 24 hour format
local FMT_NORM = 1 -- 12 hour format

AlarmClockFu.hour = 0
AlarmClockFu.minute = 0

-- Register the saved variable DB
AlarmClockFu:RegisterDB("AlarmClockFuDB")

-- Realm defaults for saved variables
AlarmClockFu:RegisterDefaults('profile', {
	offset = 0, -- the offset from server to local time
	hourfmt = FMT_NORM,
	alarmenable = 0,
	alarmmin = 0,
	alarmhr = 0
})

-- Menu Request
AlarmClockFu.OnMenuRequest = {
	type = 'group',
	args = {
		offset = {
			type = 'range',
			name = "Offset",
			desc = "Time offset from server time to local time",
			get = "GetOffset",
			set = "SetOffset",
			min = -12,
			max = 12,
			step = 1
		},
		hourfmt = {
			type = 'group',
			desc = "Select a time format",
			name = "Hour Format",
			args = {
				normalfmt = {
					type = 'toggle',
					name = "12 Hour Format",
					desc = "Select a 12 hour time format",
					set = "SetNormalFormat",
					get = "IsNormalFormat"
				},
				militaryfmt = {
					type = 'toggle',
					name = "24 Hour Format",
					desc = "Select a 24 hour time format",
					set = "SetMilitaryFormat",
					get = "IsMilitaryFormat"
				}
			}
		}
	}
}

-- Get / Set Methods
function AlarmClockFu:GetOffset()
	return self.db.profile.offset
end

function AlarmClockFu:SetOffset(v)
	self.db.profile.offset = v
end

function AlarmClockFu:IsMilitaryFormat()
	if self.db.profile.hourfmt == FMT_MIL then
		return true
	end
	
	return false
end

function AlarmClockFu:IsNormalFormat()
	if self.db.profile.hourfmt == FMT_NORM then
		return true
	end
	
	return false
end

function AlarmClockFu:SetMilitaryFormat()
	self.db.profile.hourfmt = FMT_MIL
end

function AlarmClockFu:SetNormalFormat()
	self.db.profile.hourfmt = FMT_NORM
end

-- Called when the addon is initialized
function AlarmClockFu:OnInitialize()
	self:ScheduleRepeatingEvent(self.OnUpdate, 1, self)
end

-- Formats the text displayed for the addon
function AlarmClockFu:FormatClockString()
	local suffix = ""
	local minute = self.minute
	local hour = self.hour

	hour = hour + self.db.profile.offset
	
	if hour >= 24 then
		hour = hour - 24
	elseif hour < 0 then
		hour = hour + 24
	end
	
	if self.db.profile.hourfmt == FMT_MIL then
		strtime = format(TEXT(TIME_TWENTYFOURHOURS), hour, minute)
	else
		local pm = false
		if hour >= 12 then
			pm = true
		end
		if hour > 12 then
			hour = hour - 12
		end
		if hour == 0 then
			hour = 12
		end
		if pm == false then
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " AM"
		else
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " PM"
		end
	end
	
	return strtime
end

function AlarmClockFu:FormatClockNoOffset()
	local suffix = ""
	local minute = self.minute
	local hour = self.hour
	
	if hour >= 24 then
		hour = hour - 24
	elseif hour < 0 then
		hour = hour + 24
	end
	
	if self.db.profile.hourfmt == FMT_MIL then
		strtime = format(TEXT(TIME_TWENTYFOURHOURS), hour, minute)
	else
		local pm = false
		if hour >= 12 then
			pm = true
		end
		if hour > 12 then
			hour = hour - 12
		end
		if hour == 0 then
			hour = 12
		end
		if pm == false then
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " AM"
		else
			strtime = string.gsub(string.sub(format(TEXT(TIME_TWELVEHOURAM), hour, minute), 1, 5), " ", "") .. " PM"
		end
	end
	
	return strtime
end

-- Sets the displayed text
function AlarmClockFu:OnTextUpdate()
	self.hour, self.minute = GetGameTime()
	
	self:SetText(string.format(self:FormatClockString()))
end

-- Make sure we update the text
function AlarmClockFu:OnUpdate()
	self:UpdateText()
end

local tablet = AceLibrary("Tablet-2.0")
function AlarmClockFu:OnTooltipUpdate()
	cat = tablet:AddCategory(
		'text', "Time",
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_textR2', 1,
		'child_textG2', 1,
		'child_textB2', 1
	)
	
	cat:AddLine(
		'text', "Local Time",
		'text2', self:FormatClockString()
	)
	
	cat:AddLine(
		'text', "Server Time",
		'text2', self:FormatClockNoOffset()
	)
end